==> socat-1.3.1.0.tar.bz2: socat-1.3/error.c <==
static const char _rcsid_[]="$Id: error.c,v 1.18 2003/02/05 06:29:23 gerhard Exp $";
/* Copyright Gerhard Rieger 2001-2003 */
/* Published under the GNU General Public License V.2, see file COPYING */

/* the logging subsystem */

#include "config.h"

#include <stdarg.h>
#include <stdlib.h>
#include <errno.h>
#if HAVE_SYSLOG_H
#include <syslog.h>
#endif
#include <time.h>	/* time_t, strftime() */
#include <stdio.h>
#include <string.h>
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#include "mytypes.h"
#include "compat.h"
#include "utils.h"

#include "error.h"

/* translate MSG level to SYSLOG level */
int syslevel[] = {
   LOG_DEBUG,
   LOG_INFO,
   LOG_NOTICE,
   LOG_WARNING,
   LOG_ERR,
   LOG_CRIT };

struct diag_opts {
   const char *progname;
   int msglevel;
   int exitlevel;
   int logstderr;
   int syslog;
   FILE *logfile;
   int logfacility;
} ;


struct diag_opts diagopts =
  { NULL, E_ERROR, E_ERROR, 1, 0, NULL, LOG_DAEMON } ;

static void _msg(int level, const char *buff, const char *syslp);

static struct wordent facilitynames[] = {
   {"auth",     (void *)LOG_AUTH},
#ifdef LOG_AUTHPRIV
   {"authpriv", (void *)LOG_AUTHPRIV},
#endif
#ifdef LOG_CONSOLE
   {"console",	(void *)LOG_CONSOLE},
#endif
   {"cron",     (void *)LOG_CRON},
   {"daemon",   (void *)LOG_DAEMON},
#ifdef LOG_FTP
   {"ftp",      (void *)LOG_FTP},
#endif
   {"kern",     (void *)LOG_KERN},
   {"local0",   (void *)LOG_LOCAL0},
   {"local1",   (void *)LOG_LOCAL1},
   {"local2",   (void *)LOG_LOCAL2},
   {"local3",   (void *)LOG_LOCAL3},
   {"local4",   (void *)LOG_LOCAL4},
   {"local5",   (void *)LOG_LOCAL5},
   {"local6",   (void *)LOG_LOCAL6},
   {"local7",   (void *)LOG_LOCAL7},
   {"lpr",      (void *)LOG_LPR},
   {"mail",     (void *)LOG_MAIL},
   {"news",     (void *)LOG_NEWS},
#ifdef LOG_SECURITY
   {"security",	(void *)LOG_SECURITY},
#endif
   {"syslog",   (void *)LOG_SYSLOG},
   {"user",     (void *)LOG_USER},
   {"uucp",     (void *)LOG_UUCP}
} ;


void diag_set(char what, const char *arg) {
   switch (what) {
      const struct wordent *keywd;

   case 'y': diagopts.syslog = true;
      if (arg && arg[0]) {
	 if ((keywd =
	      keyw(facilitynames, arg,
		   sizeof(facilitynames)/sizeof(struct wordent))) == NULL) {
	    Error1("unknown syslog facility \"%s\"", arg);
	 } else {
	    diagopts.logfacility = (int)keywd->desc;
	 }
      }
      openlog("socat", LOG_PID, diagopts.logfacility);
      diagopts.logstderr = false; break;
   case 'f': if ((diagopts.logfile = fopen(arg, "a")) == NULL) {
	  Error2("cannot open log file \"%s\": %s", arg, strerror(errno));
	  break;
      } else {
	 diagopts.logstderr = false; break;
      }
   case 's': diagopts.logstderr = true; break;	/* logging to stderr is default */
   case 'p': diagopts.progname = arg; break;
   case 'd': --diagopts.msglevel; break;
   case 'D': diagopts.msglevel = strtoul(arg, NULL, 10); break;
   case 'e': diagopts.exitlevel = strtoul(arg, NULL, 10); break;
   default: msg(E_ERROR, "unknown log device specification %c%s", what, arg);
   }
}

int diag_get_int(char what) {
   switch (what) {
   case 'y': return diagopts.syslog;
   case 's': return diagopts.logstderr;
   case 'd': case 'D': return diagopts.msglevel;
   case 'e': return diagopts.exitlevel;
   }
   return -1;
}

const char *diag_get_string(char what) {
   switch (what) {
   case 'p': return diagopts.progname;
   }
   return NULL;
}

/* Linux and AIX syslog format:
Oct  4 17:10:37 hostname socat[52798]: D signal(13, 1)
*/
void msg(int level, const char *format, ...) {
   time_t now;
#define BUFLEN 256
   char buff[BUFLEN], *bufp, *syslp;
   va_list ap;

   va_start(ap, format);
   if (level < diagopts.msglevel)  return;
   now = time(NULL);  if (now == (time_t)-1) {
      /* invoking msg() might create endless recursion; by hand instead */
      sprintf(buff, "cannot read time:   %s["F_pid"] E %s",
	      diagopts.progname, getpid(), strerror(errno));
      _msg(LOG_ERR, buff, strstr(buff, " E "+1));
      strcpy(buff, "unknown time        ");
   } else {
#if HAVE_STRFTIME
      strftime(buff, 21, "%Y/%m/%d %H:%M:%S ", localtime(&now));
#else
      strcpy(buff, ctime(&now));
#endif
   }
   sprintf(buff+20, "%s["F_pid"] ", diagopts.progname, getpid());
   syslp = strchr(buff+23, '\0');
   sprintf(syslp, "%c ", "DINWEF"[level]);
   bufp = strchr(syslp, '\0');
   vsnprintf(bufp, BUFLEN-35-1, format, ap);
   strcat(bufp, "\n");
   _msg(level, buff, syslp);
   if (level >= diagopts.exitlevel) {
      va_end(ap);
      exit(1);
   }
   va_end(ap);
}


static void _msg(int level, const char *buff, const char *syslp) {
   if (diagopts.logstderr) {
      fputs(buff, stderr); fflush(stderr);
   }
   if (diagopts.syslog) {
      syslog(syslevel[level], syslp);
   }
   if (diagopts.logfile) {
      fputs(buff, diagopts.logfile); fflush(diagopts.logfile);
   }
}

==> socat-1.3.2.2.tar.bz2: socat-1.3/error.c <==
/* $Id: error.c,v 1.20 2003/06/26 11:57:10 gerhard Exp $ */
/* Copyright Gerhard Rieger 2001-2003 */
/* Published under the GNU General Public License V.2, see file COPYING */

/* the logging subsystem */

#include "config.h"

#include <stdarg.h>
#include <stdlib.h>
#include <errno.h>
#if HAVE_SYSLOG_H
#include <syslog.h>
#endif
#include <time.h>	/* time_t, strftime() */
#include <stdio.h>
#include <string.h>
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#include "mytypes.h"
#include "compat.h"
#include "utils.h"

#include "error.h"

/* translate MSG level to SYSLOG level */
int syslevel[] = {
   LOG_DEBUG,
   LOG_INFO,
   LOG_NOTICE,
   LOG_WARNING,
   LOG_ERR,
   LOG_CRIT };

struct diag_opts {
   const char *progname;
   int msglevel;
   int exitlevel;
   int logstderr;
   int syslog;
   FILE *logfile;
   int logfacility;
} ;


struct diag_opts diagopts =
  { NULL, E_ERROR, E_ERROR, 1, 0, NULL, LOG_DAEMON } ;

static void _msg(int level, const char *buff, const char *syslp);

static struct wordent facilitynames[] = {
   {"auth",     (void *)LOG_AUTH},
#ifdef LOG_AUTHPRIV
   {"authpriv", (void *)LOG_AUTHPRIV},
#endif
#ifdef LOG_CONSOLE
   {"console",	(void *)LOG_CONSOLE},
#endif
   {"cron",     (void *)LOG_CRON},
   {"daemon",   (void *)LOG_DAEMON},
#ifdef LOG_FTP
   {"ftp",      (void *)LOG_FTP},
#endif
   {"kern",     (void *)LOG_KERN},
   {"local0",   (void *)LOG_LOCAL0},
   {"local1",   (void *)LOG_LOCAL1},
   {"local2",   (void *)LOG_LOCAL2},
   {"local3",   (void *)LOG_LOCAL3},
   {"local4",   (void *)LOG_LOCAL4},
   {"local5",   (void *)LOG_LOCAL5},
   {"local6",   (void *)LOG_LOCAL6},
   {"local7",   (void *)LOG_LOCAL7},
   {"lpr",      (void *)LOG_LPR},
   {"mail",     (void *)LOG_MAIL},
   {"news",     (void *)LOG_NEWS},
#ifdef LOG_SECURITY
   {"security",	(void *)LOG_SECURITY},
#endif
   {"syslog",   (void *)LOG_SYSLOG},
   {"user",     (void *)LOG_USER},
   {"uucp",     (void *)LOG_UUCP}
} ;


void diag_set(char what, const char *arg) {
   switch (what) {
      const struct wordent *keywd;

   case 'y': diagopts.syslog = true;
      if (arg && arg[0]) {
	 if ((keywd =
	      keyw(facilitynames, arg,
		   sizeof(facilitynames)/sizeof(struct wordent))) == NULL) {
	    Error1("unknown syslog facility \"%s\"", arg);
	 } else {
	    diagopts.logfacility = (int)keywd->desc;
	 }
      }
      openlog(diagopts.progname, LOG_PID, diagopts.logfacility);
      diagopts.logstderr = false; break;
   case 'f': if ((diagopts.logfile = fopen(arg, "a")) == NULL) {
	  Error2("cannot open log file \"%s\": %s", arg, strerror(errno));
	  break;
      } else {
	 diagopts.logstderr = false; break;
      }
   case 's': diagopts.logstderr = true; break;	/* logging to stderr is default */
   case 'p': diagopts.progname = arg; break;
   case 'd': --diagopts.msglevel; break;
   case 'D': diagopts.msglevel = strtoul(arg, NULL, 10); break;
   case 'e': diagopts.exitlevel = strtoul(arg, NULL, 10); break;
   default: msg(E_ERROR, "unknown log device specification %c%s", what, arg);
   }
}

int diag_get_int(char what) {
   switch (what) {
   case 'y': return diagopts.syslog;
   case 's': return diagopts.logstderr;
   case 'd': case 'D': return diagopts.msglevel;
   case 'e': return diagopts.exitlevel;
   }
   return -1;
}

const char *diag_get_string(char what) {
   switch (what) {
   case 'p': return diagopts.progname;
   }
   return NULL;
}

/* Linux and AIX syslog format:
Oct  4 17:10:37 hostname socat[52798]: D signal(13, 1)
*/
void msg(int level, const char *format, ...) {
   time_t now;
#define BUFLEN 256
   char buff[BUFLEN], *bufp, *syslp;
   va_list ap;

   va_start(ap, format);
   if (level < diagopts.msglevel)  return;
   now = time(NULL);  if (now == (time_t)-1) {
      /* invoking msg() might create endless recursion; by hand instead */
      sprintf(buff, "cannot read time:   %s["F_pid"] E %s",
	      diagopts.progname, getpid(), strerror(errno));
      _msg(LOG_ERR, buff, strstr(buff, " E "+1));
      strcpy(buff, "unknown time        ");
   } else {
#if HAVE_STRFTIME
      strftime(buff, 21, "%Y/%m/%d %H:%M:%S ", localtime(&now));
#else
      strcpy(buff, ctime(&now));
#endif
   }
   sprintf(buff+20, "%s["F_pid"] ", diagopts.progname, getpid());
   syslp = strchr(buff+23, '\0');
   sprintf(syslp, "%c ", "DINWEF"[level]);
   bufp = strchr(syslp, '\0');
   vsnprintf(bufp, BUFLEN-35-1, format, ap);
   strcat(bufp, "\n");
   _msg(level, buff, syslp);
   if (level >= diagopts.exitlevel) {
      va_end(ap);
      exit(1);
   }
   va_end(ap);
}


static void _msg(int level, const char *buff, const char *syslp) {
   if (diagopts.logstderr) {
      fputs(buff, stderr); fflush(stderr);
   }
   if (diagopts.syslog) {
      syslog(syslevel[level], syslp);
   }
   if (diagopts.logfile) {
      fputs(buff, diagopts.logfile); fflush(diagopts.logfile);
   }
}
